local class = require "yale/class"
local U = require "yale/util"

local EN = require "yale/entity"
local ENM = require "enemy"
local Bul = require "bullet"

local atan2 = math.atan2
local cos = memcos
local sin = memsin
local pi = math.pi
local setColor = love.graphics.setColor
local circle = love.graphics.circle
local line = love.graphics.line

-- your gunner

class "gunner" ("enemy") {
	init = function(o)
		EN.init(o)
		o.max_health = 1
		o.health = o.max_health
		o.shot_stamp = 0
		o.shot_wait = 1
	end
}

local function init_physics(e)
	e.body = e:add_body("main", 0, 0)
	e.body:setAngularDamping(32)
	e.body:setLinearDamping(10)
	e.shape = e:add_shape("main", "circle", "main", 0, 0, 7)
	e.shape:setCategory(2) -- enemy category - evens
	e.shape:setMask(4, 16) -- enemy bullets. also, enemies can pass through the invisible walls of the WORLD
	e:set_body_masses()
end

local function update(e, dt)
	ENM.update(e, dt)
	local px, py = e:get_player_pos()
	local ride = (px*px+py*py)^.5 - 100
	local ride = ride > 100 and 100 or ride

	local nx, ny = U.normalize(px, py)
	if e.ccw then nx, ny = nx * ride - 100*ny, ny * ride + 100*nx
	else nx, ny = nx * ride + 100*ny, ny * ride - 100*nx
	end
	local fx, fy = U.normalize(nx, ny)
	e.body:applyForce(fx * 50, fy * 50)
	
	-- track angle
	local theta = e.body:getAngle()
	local theta2 = atan2(py, px)
	local pdtheta = (theta2 - theta) % (2 * pi)
	local ndtheta = 2 * pi - pdtheta
	if pdtheta < ndtheta then
		e.body:applyTorque(pdtheta * 32)
	else
		e.body:applyTorque(-ndtheta * 32)
	end
	e:shoot(dt)
end

local function draw(e)
	local hrat = e.health/e.max_health
	setColor(U.hsl2rgb(hrat*.5, .5, hrat*.15))
	local cx, cy = e.shape:getWorldCenter()
	circle("fill", cx, cy, e.shape:getRadius(), 16)
	setColor(255, 255*hrat, 255*hrat)
	circle("line", cx, cy, e.shape:getRadius(), 16)
	

	local theta = e.body:getAngle()
	local x, y = e.body:getX(), e.body:getY()
	local dx, dy = cos(theta), sin(theta)
	line(x, y, x+dx*7.5, y+dy*7.5)
end

local function shoot(e, dt)
	local shot_stamp = e.shot_stamp + dt
	local shot_wait = e.shot_wait
	
	while shot_stamp > 0 do
		Bul {
			world = e.world,
			x = e.x,
			y = e.y,
			cat = 4,
			angle = e.body:getAngle(),
			fcolor = { 255, 0, 0 },
			I = .8,
			dam = .01
		}
		
		shot_stamp = shot_stamp - shot_wait
	end
	
	e.shot_stamp = shot_stamp
end

local function collision_add(e, p, c, switch)
	ENM.collision_add(e, p, c, switch)
	e.ccw = not e.ccw
end

local G = class.gunner
G.init_physics = init_physics
G.update = update
G.draw = draw
G.shoot = shoot
G.collision_add = collision_add
return G
