local class = require "yale/class"
local U = require "yale/util"

local EN = require "yale/entity"
local M = require "missile"


local cos = memcos
local sin = memsin
local setColor = love.graphics.setColor
local circle = love.graphics.circle
local line = love.graphics.line

require "enemies/gunner"

class "homing_gunner" ("gunner") {
	init = function(o)
		EN.init(o)
		o.max_health = 2
		o.health = o.max_health
		o.shot_stamp = 0
		o.shot_wait = 2
	end
}

local function init_physics(e)
	e.body = e:add_body("main", 0, 0)
	e.body:setAngularDamping(32)
	e.body:setLinearDamping(10)
	e.shape = e:add_shape("main", "circle", "main", 0, 0, 10)
	e.shape:setCategory(2) -- enemy category - evens
	e.shape:setMask(4, 16) -- enemy bullets. also, enemies can pass through the invisible walls of the WORLD
	e:set_body_masses()
end

local function draw(e)
	local hrat = e.health/e.max_health
	setColor(U.hsl2rgb(hrat*.5, .75, .05-hrat*.05))
	local cx, cy = e.shape:getWorldCenter()
	circle("fill", cx, cy, e.shape:getRadius(), 16)
	setColor(255, 255*hrat, 255*hrat)
	circle("line", cx, cy, e.shape:getRadius(), 16)
	

	local theta = e.body:getAngle()
	local x, y = e.body:getX(), e.body:getY()
	local dx, dy = cos(theta), sin(theta)
	line(x, y, x+dx*10, y+dy*10)
end

local function shoot(e, dt)
	local shot_stamp = e.shot_stamp + dt
	local shot_wait = e.shot_wait
	
	while shot_stamp > 0 do
		M {
			world = e.world,
			x = e.x,
			y = e.y,
			cat = 6,
			mask = {2, 4, 6},
			angle = e.body:getAngle(),
			fcolor = { 255, 0, 0 },
			I = 5,
			dam = .2,
			target = player
		}
		
		shot_stamp = shot_stamp - shot_wait
	end
	
	e.shot_stamp = shot_stamp
end

local HG = class.homing_gunner
HG.init_physics = init_physics
HG.draw = draw
HG.shoot = shoot
return HG
