local class = require "yale/class"
local U = require "yale/util"

local Ex = require "explosion"

local setColor = love.graphics.setColor
local circle = love.graphics.circle

local EN = require "yale.entity"

-- base enemy entity
class "enemy" ("entity") {
	init = function(o)
		EN.init(o)
		o.max_health = .3
		o.health = o.max_health
	end
}

-- percepts available to enemies
-- player's position relative to enemy
local function get_player_pos(e)
	--if player.dead then return 0, 0 end
	return player.x - e.x, player.y - e.y
end

-- regular stuff
local function init_physics(e)
	e.body = e:add_body("main", 0, 0)
	e.body:setAngularDamping(32)
	e.body:setLinearDamping(10)
	e.shape = e:add_shape("main", "circle", "main", 0, 0, 6)
	e.shape:setCategory(2) -- enemy category - evens
	e.shape:setMask(4, 16) -- enemy bullets. also, enemies can pass through the invisible walls of the WORLD
	e:set_body_masses()
end
local function update(e, dt)
	-- are you supposed to die? if yes, please commence dying here:
	if e.health <= 0 then
		e.body:setPosition(-10000, -10000)
		e.dead = true
		
		-- explosion
		Ex{ world = e.world, x = e.x, y = e.y, speed = 100*e.max_health }
		return
	end
	-- position update
	e.x, e.y = e.body:getPosition()
	-- stop when player is dead
	if player.dead then e.paused=true end
end
local function draw(e)
	local hrat = e.health/e.max_health
	setColor(U.hsl2rgb(hrat*.5, .5, hrat*.15))
	local cx, cy = e.shape:getWorldCenter()
	circle("fill", cx, cy, e.shape:getRadius(), 16)
	setColor(255, 255*hrat, 255*hrat)
	circle("line", cx, cy, e.shape:getRadius(), 16)
end
local function damage(e, d)
	e.health = e.health - d
end
local function collision_add(e, p, c, switch)
	if p and p:is_a "player" then
		local cnx, cny = c:getNormal()
		local cpx, cpy = c:getPosition()
		if switch then
			cnx = -cnx
			cny = -cny
		end
		-- self destruct for maxx damage!
		p.body:applyImpulse(4*e.health*cnx, 4*e.health*cny, cpx, cpy)
		p:damage(e.health)
		e:damage(e.health)
	end
end

-- export stupidity
local E = class.enemy
E.get_player_pos = get_player_pos
E.init_physics = init_physics
E.update = update
E.draw = draw
E.damage = damage
E.collision_add = collision_add
return E
