local class = require "yale/class"
local U = require "yale/util"

local setColor = love.graphics.setColor
local polygon = love.graphics.polygon

local EN = require "yale.entity"
local Ex = require "explosion"

local sin = memsin
local cos = memcos
local unpack = unpack

class "missile" ("entity") {}

local function init_physics(b)
	local body = b:add_body("main", 0, 0)
	local shape = b:add_shape("main", "rectangle", "main", 0, 0, 8, 8, b.angle)
	body:setLinearDamping(1)
	body:setAngularVelocity(20)
	shape:setCategory(b.cat) -- set category
	shape:setMask(unpack(b.mask)) -- mask bullets categories
	body:setMassFromShapes()
	body:setBullet(true)
	body:applyImpulse(b.I*cos(b.angle), b.I*sin(b.angle))
	b.body = body
	b.shape = shape
end

local function update(b, dt)
	if b.body:isFrozen() then b.dead = true end
	b.x, b.y = b.body:getPosition()
	local tx, ty = b.target.x - b.x, b.target.y - b.y
	local fx, fy = U.normalize(tx, ty)
	b.body:applyForce(fx * b.I, fy * b.I)
end

local function draw(b)
	setColor(unpack(b.fcolor))
	polygon("fill", b.shape:getPoints())
	setColor(255, 255, 255)
	polygon("line", b.shape:getPoints())
end

local function collision_add(b, e, c, switch)
	local df = e and e.damage or nil
	if df then
		-- damage the fuckerr
		df(e, b.dam)
	end
	b.body:setPosition(-10000, -10000)
	b.dead = true
	local x, y = c:getPosition()
	-- explosion
	Ex{ world = b.world, x = x, y = y, speed = 100*b.dam }
end

local M = class.missile
M.init_physics = init_physics
M.update = update
M.draw = draw
M.collision_add = collision_add
return M
