local class = require "yale/class"

local getScreenWidth = love.graphics.getWidth
local getScreenHeight = love.graphics.getHeight
local newWorld = love.physics.newWorld

-- collision callbacks
-- assumes data points to containing entity object
local function add_cb(a, b, c)
	local acb = a and a.collision_add
	local bcb = b and b.collision_add
	if acb then acb(a, b, c, false) end
	if bcb then bcb(b, a, c, true) end
end
local function persist_cb(a, b, c)
	local acb = a and a.collision_persist
	local bcb = b and b.collision_persist
	if acb then acb(a, b, c, false) end
	if bcb then bcb(b, a, c, true) end
end
local function remove_cb(a, b, c)
	local acb = a and a.collision_remove
	local bcb = b and b.collision_remove
	if acb then acb(a, b, c, false) end
	if bcb then bcb(b, a, c, true) end
end
local function result_cb(a, b, c)
	local acb = a and a.collision_result
	local bcb = b and b.collision_result
	if acb then acb(a, b, c, false) end
	if bcb then bcb(b, a, c, true) end
end

-- world class

class "world" (nil) {
	init = function(o)
		-- world AABB
		local startx = o.startx or -getScreenWidth()
		local starty = o.starty or -getScreenHeight()
		local w = o.w or getScreenWidth()*3
		local h = o.h or getScreenHeight()*3
		-- world gravity
		local gx = o.gx or 0
		local gy = o.gy or 0
		local m = o.px_per_m
		
		local scalefactor = 30 / m
		local physics = newWorld(startx * scalefactor, starty * scalefactor,
			(startx + w) * scalefactor, (starty + h) * scalefactor,
			gx, gy, true)
		
		physics:setMeter(m)
		physics:setCallbacks(add_cb, persist_cb, remove_cb, result_cb)
		
		o.startx = startx
		o.starty = starty
		o.w = w
		o.h = h
		o.gx = gx
		o.gy = gy
		o.paused = false
		o.physics = physics
		o.entities = {}
		o.timestamp = 0
	end
}

-- general stuff
local function update(w, dt)
	if w.paused then return end
	w.physics:update(dt)
	
	local entities = w.entities
	
	for i=1,#entities do
		local e = entities[i]
		if not e.paused then
			e:update(dt)
		end
	end
	
	local shift = 0
	for i=1,#entities do
		local e = entities[i]
		if shift > 0 then
			entities[i-shift] = e
			entities[i] = nil
		end
		
		if e.dead then
			entities[i] = nil
			shift = shift + 1
		end
	end
end
local function draw(w)
	local entities = w.entities
	for i=1,#entities do
		local e = entities[i]
		if not e.invisible then e:draw() end
	end
end

-- export
local W = class.world
W.px_per_m = 30
W.update = update
W.draw = draw
return W
